@@ -37,6 +37,8 @@ except (ImportError, AttributeError):
 
 import chardet
 import collections
+import datetime
+import email.utils
 import re
 import subprocess
 import sys
@@ -381,7 +383,7 @@ class Deb822(Deb822Dict):
                     yield paragraph
 
         else:
-            if isinstance(sequence, six.string_types):
+            if isinstance(sequence, six.string_types + (six.binary_type,)):
                 sequence = sequence.splitlines()
             iterable = iter(sequence)
             while True:
@@ -905,7 +907,7 @@ class PkgRelation(object):
             r'$')
     __comma_sep_RE = re.compile(r'\s*,\s*')
     __pipe_sep_RE = re.compile(r'\s*\|\s*')
-    __blank_sep_RE = re.compile(r'\s*')
+    __blank_sep_RE = re.compile(r'\s+')
     __restriction_sep_RE = re.compile(r'>\s*<')
     __restriction_RE = re.compile(
             r'(?P<enabled>\!)?'
@@ -1598,6 +1600,128 @@ class RestrictedWrapper(object):
         return self.__data.dump(*args, **kwargs)
 
 
+class Removals(Deb822):
+    """Represent an ftp-master removals.822 file
+
+    Removal of packages from the archive are recorded by ftp-masters.
+    See https://ftp-master.debian.org/#removed
+
+    Note: this API is experimental and backwards-incompatible changes might be
+    required in the future. Please use it and help us improve it!
+    """
+    __sources_line_re = re.compile(r'\s*'
+                            r'(?P<package>.+?)'
+                            r'_'
+                            r'(?P<version>[^\s]+)'
+                            r'\s*'
+                        )
+    __binaries_line_re = re.compile(r'\s*'
+                            r'(?P<package>.+?)'
+                            r'_'
+                            r'(?P<version>[^\s]+)'
+                            r'\s+'
+                            r'\[(?P<archs>.+)\]'
+                        )
+
+    @property
+    def date(self):
+        """ a datetime object for the removal action """
+        ts = email.utils.mktime_tz(email.utils.parsedate_tz(self['date']))
+        return datetime.datetime.fromtimestamp(ts)
+
+    @property
+    def bug(self):
+        """ list of bug numbers that had requested the package removal
+
+        The bug numbers are returned as integers.
+
+        Note: there is normally only one entry in this list but there may be
+        more than one.
+        """
+        if 'bug' not in self:
+            return []
+        return [int(b) for b in self['bug'].split(",")]
+
+    @property
+    def also_wnpp(self):
+        """ list of WNPP bug numbers closed by the removal
+
+        The bug numbers are returned as integers.
+        """
+        if 'also-wnpp' not in self:
+            return []
+        return [int(b) for b in self['also-wnpp'].split(" ")]
+
+    @property
+    def also_bugs(self):
+        """ list of bug numbers in the package closed by the removal
+
+        The bug numbers are returned as integers.
+
+        Removal of a package implicitly also closes all bugs associated with
+        the package.
+        """
+        if 'also-bugs' not in self:
+            return []
+        return [int(b) for b in self['also-bugs'].split(" ")]
+
+    @property
+    def sources(self):
+        """ list of source packages that were removed
+
+        A list of dicts is returned, each dict has the form:
+            {
+                'source': 'some-package-name',
+                'version': '1.2.3-1'
+            }
+
+        Note: There may be no source packages removed at all if the removal is
+        only of a binary package. An empty list is returned in that case.
+        """
+        if hasattr(self, '_sources'):
+            return self._sources
+
+        s = []
+        if 'sources' in self:
+            for line in self['sources'].splitlines():
+                matches = self.__sources_line_re.match(line)
+                if matches:
+                    s.append({
+                            'source': matches.group('package'),
+                            'version': matches.group('version'),
+                        })
+        self._sources = s
+        return s
+
+    @property
+    def binaries(self):
+        """ list of binary packages that were removed
+
+        A list of dicts is returned, each dict has the form:
+            {
+                'package': 'some-package-name',
+                'version': '1.2.3-1',
+                'architectures': set(['i386', 'amd64'])
+            }
+        """
+        if hasattr(self, '_binaries'):
+            return self._binaries
+
+        b = []
+        if 'binaries' in self:
+            for line in self['binaries'].splitlines():
+                matches = self.__binaries_line_re.match(line)
+                if matches:
+                    b.append({
+                            'package': matches.group('package'),
+                            'version': matches.group('version'),
+                            'architectures':
+                                    set(matches.group('archs').split(', ')),
+                        })
+        self._binaries = b
+        return b
+
+
 class _CaseInsensitiveString(str):
     """Case insensitive string.
     """
