@@ -19,6 +19,7 @@
 
 from __future__ import absolute_import
 
+import email.utils
 import io
 import os
 import re
@@ -465,6 +466,16 @@ class TestDeb822(unittest.TestCase):
                             for d in deb822.Deb822.iter_paragraphs(text)])
             self.assertEqual(2, count)
 
+    def test_iter_paragraphs_bytes(self):
+        text = (UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
+        if six.PY2:
+            binary = text
+        else:
+            binary = text.encode('utf-8')
+
+        for d in deb822.Deb822.iter_paragraphs(binary):
+            self.assertWellParsed(d, PARSED_PACKAGE)
+
     def test_iter_paragraphs_with_extra_whitespace(self):
         """ Paragraphs not elided when stray whitespace is between
 
@@ -951,6 +962,30 @@ Description: python modules to work with Debian-related data formats
             changes = deb822.Changes(f)
         self.assertEqual('python-debian', changes['Source'])
 
+    def test_removals(self):
+        with open('test_removals.822') as f:
+            removals = deb822.Removals.iter_paragraphs(f)
+            r = next(removals)
+            self.assertEqual(r['suite'], 'unstable')
+            self.assertEqual(r['date'], u'Wed, 01 Jan 2014 17:03:54 +0000')
+            self.assertEqual(r.date.strftime('%s'), '1388595834')
+            self.assertEqual(len(r.binaries), 1)
+            self.assertEqual(r.binaries[0]['package'], 'libzoom-ruby')
+            self.assertEqual(r.binaries[0]['version'], '0.4.1-5')
+            self.assertEqual(r.binaries[0]['architectures'], set(['all']))
+            r = next(removals)
+            self.assertEqual(len(r.binaries), 3)
+            r = next(removals)
+            self.assertEqual(r['bug'], '753912')
+            self.assertEqual(r.bug, [753912])
+            self.assertEqual(r.also_wnpp, [123456])
+            r = next(removals)
+            self.assertEqual(r.binaries[0]['architectures'],
+                             set(['amd64', 'armel', 'armhf', 'hurd-i386',
+                                  'i386', 'kfreebsd-amd64', 'kfreebsd-i386',
+                                  'mips', 'mipsel', 'powerpc', 's390x',
+                                  'sparc']))
+
 
 class TestPkgRelations(unittest.TestCase):
     # TODO(jsw): Stop overriding this for Python versions that actually include
